@@ -45,8 +45,6 @@ using namespace std;
 
 #define dout_subsys ceph_subsys_mon
 
-extern CompatSet get_ceph_mon_feature_compat_set();
-
 Monitor *mon = NULL;
 
 void handle_mon_signal(int signum)
@@ -217,9 +215,6 @@ int main(int argc, const char **argv)
     return 0;
   }
 
-  CompatSet mon_features = get_ceph_mon_feature_compat_set();
-  CompatSet ondisk_features;
-
   MonitorStore store(g_conf->mon_data);
   err = store.mount();
   if (err < 0) {
@@ -239,29 +234,12 @@ int main(int argc, const char **argv)
     exit(1);
   }
 
-  bufferlist features;
-  store.get_bl_ss(features, COMPAT_SET_LOC, 0);
-  if (features.length() == 0) {
-    cerr << "WARNING: mon fs missing feature list.\n"
-	 << "Assuming it is old-style and introducing one." << std::endl;
-    //we only want the baseline ~v.18 features assumed to be on disk.
-    //If new features are introduced this code needs to disappear or
-    //be made smarter.
-    ondisk_features = get_ceph_mon_feature_compat_set();
-  } else {
-    bufferlist::iterator it = features.begin();
-    ondisk_features.decode(it);
-  }
-  
-  if (!mon_features.writeable(ondisk_features)) {
-    cerr << "monitor executable cannot read disk! Missing features: "
-	 << std::endl;
-    CompatSet diff = mon_features.unsupported(ondisk_features);
-    //NEEDS_COMPATSET_ITER
+  err = Monitor::check_features(&store);
+  if (err < 0) {
+    cerr << "error checking features: " << cpp_strerror(err) << std::endl;
     exit(1);
   }
 
-
   // inject new monmap?
   if (!inject_monmap.empty()) {
     bufferlist bl;
@@ -306,14 +284,14 @@ int main(int argc, const char **argv)
   {
     bufferlist mapbl;
     bufferlist latest;
-    store.get_bl_ss(latest, "monmap", "latest");
+    store.get_bl_ss_safe(latest, "monmap", "latest");
     if (latest.length() > 0) {
       bufferlist::iterator p = latest.begin();
       version_t v;
       ::decode(v, p);
       ::decode(mapbl, p);
     } else {
-      store.get_bl_ss(mapbl, "mkfs", "monmap");
+      store.get_bl_ss_safe(mapbl, "mkfs", "monmap");
       if (mapbl.length() == 0) {
 	cerr << "mon fs missing 'monmap/latest' and 'mkfs/monmap'" << std::endl;
 	exit(1);
@@ -352,6 +330,8 @@ int main(int argc, const char **argv)
     pick_addresses(g_ceph_context);
     if (!g_conf->public_addr.is_blank_ip()) {
       ipaddr = g_conf->public_addr;
+      if (ipaddr.get_port() == 0)
+	ipaddr.set_port(CEPH_MON_PORT);
     } else {
       MonMap tmpmap;
       int err = tmpmap.build_initial(g_ceph_context, cerr);
@@ -383,7 +363,8 @@ int main(int argc, const char **argv)
     CEPH_FEATURE_UID |
     CEPH_FEATURE_NOSRCADDR |
     CEPH_FEATURE_MONCLOCKCHECK |
-    CEPH_FEATURE_PGID64;
+    CEPH_FEATURE_PGID64 |
+    CEPH_FEATURE_MSG_AUTH;
   messenger->set_default_policy(Messenger::Policy::stateless_server(supported, 0));
   messenger->set_policy(entity_name_t::TYPE_MON,
                         Messenger::Policy::lossless_peer_reuse(supported,
@@ -400,17 +381,17 @@ int main(int argc, const char **argv)
 
 
   // throttle client traffic
-  Throttle client_throttler(g_ceph_context, "mon_client_bytes",
-			    g_conf->mon_client_bytes);
-  messenger->set_policy_throttler(entity_name_t::TYPE_CLIENT, &client_throttler);
+  Throttle *client_throttler = new Throttle(g_ceph_context, "mon_client_bytes",
+					    g_conf->mon_client_bytes);
+  messenger->set_policy_throttler(entity_name_t::TYPE_CLIENT, client_throttler);
 
   // throttle daemon traffic
   // NOTE: actual usage on the leader may multiply by the number of
   // monitors if they forward large update messages from daemons.
-  Throttle daemon_throttler(g_ceph_context, "mon_daemon_bytes",
-			    g_conf->mon_daemon_bytes);
-  messenger->set_policy_throttler(entity_name_t::TYPE_OSD, &daemon_throttler);
-  messenger->set_policy_throttler(entity_name_t::TYPE_MDS, &daemon_throttler);
+  Throttle *daemon_throttler = new Throttle(g_ceph_context, "mon_daemon_bytes",
+					    g_conf->mon_daemon_bytes);
+  messenger->set_policy_throttler(entity_name_t::TYPE_OSD, daemon_throttler);
+  messenger->set_policy_throttler(entity_name_t::TYPE_MDS, daemon_throttler);
 
   cout << "starting " << g_conf->name << " rank " << rank
        << " at " << ipaddr
@@ -425,10 +406,13 @@ int main(int argc, const char **argv)
   // start monitor
   mon = new Monitor(g_ceph_context, g_conf->name.get_id(), &store, messenger, &monmap);
 
+  err = mon->preinit();
+  if (err < 0)
+    return 1;
+
   global_init_daemonize(g_ceph_context, 0);
   common_init_finish(g_ceph_context);
   global_init_chdir(g_ceph_context);
-  messenger->start();
 
   // set up signal handlers, now that we've daemonized/forked.
   init_async_signal_handler();
@@ -436,16 +420,24 @@ int main(int argc, const char **argv)
   register_async_signal_handler_oneshot(SIGINT, handle_mon_signal);
   register_async_signal_handler_oneshot(SIGTERM, handle_mon_signal);
 
+  messenger->start();
+
   mon->init();
+
   messenger->wait();
 
   unregister_async_signal_handler(SIGHUP, sighup_handler);
   unregister_async_signal_handler(SIGINT, handle_mon_signal);
   unregister_async_signal_handler(SIGTERM, handle_mon_signal);
 
+  shutdown_async_signal_handler();
+
   store.umount();
   delete mon;
   delete messenger;
+  delete client_throttler;
+  delete daemon_throttler;
+  g_ceph_context->put();
 
   // cd on exit, so that gmon.out (if any) goes into a separate directory for each node.
   char s[20];
