@@ -72,6 +72,10 @@ public:
   __u32&         sec_ref()  { return tv.tv_sec; }
   __u32&         nsec_ref() { return tv.tv_nsec; }
 
+  uint64_t to_nsec() const {
+    return (uint64_t)tv.tv_nsec + (uint64_t)tv.tv_sec * 1000000000ull;
+  }
+
   void copy_to_timeval(struct timeval *v) const {
     v->tv_sec = tv.tv_sec;
     v->tv_usec = tv.tv_nsec/1000;
