@@ -96,7 +96,7 @@ void AuthMonitor::create_initial()
 
   KeyRing keyring;
   bufferlist bl;
-  mon->store->get_bl_ss(bl, "mkfs", "keyring");
+  mon->store->get_bl_ss_safe(bl, "mkfs", "keyring");
   bufferlist::iterator p = bl.begin();
   ::decode(keyring, p);
 
@@ -286,26 +286,16 @@ uint64_t AuthMonitor::assign_global_id(MAuth *m, bool should_increase_max)
     dout(10) << "next_global_id should be " << next_global_id << dendl;
   }
 
-  while (next_global_id >= max_global_id) {
-    if (!mon->is_leader()) {
-      dout(10) << "not the leader, requesting more ids from leader" << dendl;
-      int leader = mon->get_leader();
-      MMonGlobalID *req = new MMonGlobalID();
-      req->old_max_id = max_global_id;
-      mon->messenger->send_message(req, mon->monmap->get_inst(leader));
-      paxos->wait_for_commit(new C_RetryMessage(this, m));
+  if (next_global_id >= max_global_id) {
+    if (!mon->is_leader() || !should_increase_max) {
       return 0;
-    } else {
-      if (!should_increase_max)
-        return 0;
-
-      dout(10) << "increasing max_global_id" << dendl;
+    }
+    while (next_global_id >= max_global_id) {
       increase_max_global_id();
     }
   }
 
   last_allocated_id = next_global_id;
-
   return next_global_id;
 }
 
@@ -372,15 +362,22 @@ bool AuthMonitor::prep_auth(MAuth *m, bool paxos_writable)
   if (!s->global_id) {
     s->global_id = assign_global_id(m, paxos_writable);
     if (!s->global_id) {
-      s->put();
-
       delete s->auth_handler;
       s->auth_handler = NULL;
+      s->put();
 
-      //we don't m->put() here because assign_global_id has queued it up
-      if (mon->is_leader())
-	return false;
-      return true;
+      if (!mon->is_leader()) {
+	dout(10) << "not the leader, requesting more ids from leader" << dendl;
+	int leader = mon->get_leader();
+	MMonGlobalID *req = new MMonGlobalID();
+	req->old_max_id = max_global_id;
+	mon->messenger->send_message(req, mon->monmap->get_inst(leader));
+	paxos->wait_for_commit(new C_RetryMessage(this, m));
+	return true;
+      }
+
+      assert(!paxos_writable);
+      return false;
     }
   }
 
@@ -552,7 +549,8 @@ void AuthMonitor::import_keyring(KeyRing& keyring)
     auth_inc.name = p->first;
     auth_inc.auth = p->second; 
     auth_inc.op = KeyServerData::AUTH_INC_ADD;
-    dout(10) << " importing " << auth_inc.name << " " << auth_inc.auth << dendl;
+    dout(10) << " importing " << auth_inc.name << dendl;
+    dout(30) << "    " << auth_inc.auth << dendl;
     push_cephx_inc(auth_inc);
   }
 }
@@ -630,7 +628,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
       for (unsigned i=3; i+1<m->cmd.size(); i += 2)
 	::encode(m->cmd[i+1], auth_inc.auth.caps[m->cmd[i]]);
 
-      dout(10) << " importing " << auth_inc.name << " " << auth_inc.auth << dendl;
+      dout(10) << " importing " << auth_inc.name << dendl;
+      dout(30) << "    " << auth_inc.auth << dendl;
       push_cephx_inc(auth_inc);
 
       ss << "added key for " << auth_inc.name;
@@ -720,12 +719,11 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 	err = -EINVAL;
 	goto done;
       }
-      if (!mon->key_server.contains(auth_inc.name)) {
+      if (!mon->key_server.get_auth(auth_inc.name, auth_inc.auth)) {
         ss << "couldn't find entry " << auth_inc.name;
         err = -ENOENT;
         goto done;
       }
-      mon->key_server.get_auth(auth_inc.name, auth_inc.auth);
 
       map<string,bufferlist> newcaps;
       for (unsigned i=3; i+1<m->cmd.size(); i += 2)
@@ -743,7 +741,12 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
     else if (m->cmd[1] == "del" && m->cmd.size() >= 3) {
       string name = m->cmd[2];
       KeyServerData::Incremental auth_inc;
-      auth_inc.name.from_str(name);
+      bool r = auth_inc.name.from_str(name);
+      if (r == false) {
+	ss << "bad entity name " << name;
+	err = -EINVAL;
+	goto done;
+      }
       if (!mon->key_server.contains(auth_inc.name)) {
         ss << "couldn't find entry " << name;
         err = -ENOENT;
