@@ -28,6 +28,8 @@
 
 #include "messages/MMonSubscribe.h"
 
+#include "common/SimpleRNG.h"
+
 #include <memory>
 
 class MonMap;
@@ -38,6 +40,8 @@ class MMonSubscribeAck;
 class MAuthReply;
 class MAuthRotating;
 class LogClient;
+class AuthSupported;
+class AuthAuthorizeHandlerRegistry;
 class AuthMethodList;
 
 enum MonClientState {
@@ -58,6 +62,8 @@ private:
   string cur_mon;
   Connection *cur_con;
 
+  SimpleRNG rng;
+
   EntityName entity_name;
 
   entity_addr_t my_addr;
@@ -65,6 +71,11 @@ private:
   Mutex monc_lock;
   SafeTimer timer;
   Finisher finisher;
+
+  // Added to support session signatures.  PLR
+
+  AuthAuthorizeHandlerRegistry *authorize_handler_registry;
+
   bool initialized;
 
   LogClient *log_client;
@@ -112,6 +123,7 @@ private:
 
   list<Message*> waiting_for_session;
 
+  string _pick_random_mon();
   void _finish_hunting();
   void _reopen_session();
   void _pick_new_mon();
@@ -245,16 +257,6 @@ public:
 
   // version requests
 public:
-  /**
-   * check if we have the latest version of a map
-   *
-   * @param map name of map (e.g., 'osdmap')
-   * @param cur_ver version we have
-   * @param onfinish completion
-   * @return (via context) 1 if cur_ver is the latest, 0 if it is not, -EAGAIN if we need to retry
-   */
-  void is_latest_map(string map, version_t cur_ver, Context *onfinish);
-
   /**
    * get latest known version(s) of cluster map
    *
