@@ -33,19 +33,18 @@
 #include "FlatIndex.h"
 #include "CollectionIndex.h"
 
-int do_getxattr(const char *fn, const char *name, void *val, size_t size);
-int do_setxattr(const char *fn, const char *name, const void *val, size_t size);
+#include "chain_xattr.h"
 
 static int set_version(const char *path, uint32_t version) {
   bufferlist bl;
   ::encode(version, bl);
-  return do_setxattr(path, "user.cephos.collection_version", bl.c_str(), 
+  return chain_setxattr(path, "user.cephos.collection_version", bl.c_str(), 
 		     bl.length());
 }
 
 static int get_version(const char *path, uint32_t *version) {
   bufferptr bp(PATH_MAX);
-  int r = do_getxattr(path, "user.cephos.collection_version", 
+  int r = chain_getxattr(path, "user.cephos.collection_version", 
 		      bp.c_str(), bp.length());
   if (r < 0) {
     if (r != -ENOENT) {
@@ -83,7 +82,7 @@ int IndexManager::init_index(coll_t c, const char *path, uint32_t version) {
 
 int IndexManager::build_index(coll_t c, const char *path, Index *index) {
   int r;
-  if (g_conf->filestore_update_collections) {
+  if (upgrade) {
     // Need to check the collection generation
     uint32_t version = 0;
     r = get_version(path, &version);
@@ -97,7 +96,8 @@ int IndexManager::build_index(coll_t c, const char *path, Index *index) {
       return 0;
     }
     case CollectionIndex::HASH_INDEX_TAG: // fall through
-    case CollectionIndex::HASH_INDEX_TAG_2: {
+    case CollectionIndex::HASH_INDEX_TAG_2: // fall through
+    case CollectionIndex::HOBJECT_WITH_POOL: {
       // Must be a HashIndex
       *index = Index(new HashIndex(c, path, g_conf->filestore_merge_threshold,
 				   g_conf->filestore_split_multiple, version), 
@@ -111,7 +111,7 @@ int IndexManager::build_index(coll_t c, const char *path, Index *index) {
     // No need to check
     *index = Index(new HashIndex(c, path, g_conf->filestore_merge_threshold,
 				 g_conf->filestore_split_multiple,
-				 CollectionIndex::HASH_INDEX_TAG_2), 
+				 CollectionIndex::HOBJECT_WITH_POOL), 
 		   RemoveOnDelete(c, this));
     return 0;
   }
